local a = {}
a.__index = a

function a:new(name)
	self = setmetatable({}, self)
	self.source  = love.audio.newSource(name, 'stream')
	
	local bytespersample    = self.source:getChannelCount() * 2 -- 16-bit sound
	local max_fft_frequency = 1400
	
	self.decoder = love.sound.newDecoder(name, max_fft_frequency * bytespersample)
	
	local meta = getmetatable(self.source)
	for k, v in pairs(meta.__index) do
		self[k] = function(self, ...) return self.source[k](self.source, ...) end
	end
	
	self.source:setLooping(true)
	return self
end

function a:getBytes(dt)
	local l = self.source:tell()
	l = l - dt
	l = l < 0 and 0 or l
	self.decoder:seek(l)
	return self.decoder:decode()
end

local o = a:new('Twistboy - Black Light.mp3')
o:play()

love.graphics.setLineStyle('rough')
love.graphics.setLineJoin( 'none' )

function love.filedropped(f)
	local n = f:getFilename():lower()
	if n:find("%.mp3$") or n:find("%.ogg$") then
		o:stop()
		o = a:new(f)
		o:play()
	end
end

function love.draw()
	love.window.setTitle(love.timer.getFPS())
	love.graphics.setColor(1, 1, 1)
	love.graphics.print('Drag and drop an mp3/ogg file')

	local graph = {}
	local data = o:getBytes(love.timer.getDelta())
	if not data then return end
	
	local w, h = love.graphics.getDimensions()
	local gp = h / 2
	local gh = h / 2
	
	local n = -1

	local max = data:getSampleCount()
	
	local step = w / max
	for i = 1, max do
		local p = i * 2
		graph[p - 1] = i * step
		graph[p]     = data:getSample(i - 1, 1) * gh + gp
	end
	love.graphics.setColor(.2, 1, .2, .5)
	love.graphics.line(graph)
	
	local f = {}
	for i = 1, max do
		local d = data:getSample(i - 1, 2)
		graph[i * 2]     = d * gh + gp
		f[i] = d
	end
	
	love.graphics.setColor(.2, .2, 1, .5)
	love.graphics.line(graph)
	
	local fft = require'fft'(f)
	
	
	love.graphics.setColor(1, 1, 1)
	for i = 1, #fft do
		local magnitude = (fft[i].i^2 + fft[i].r^2)^.5 
		graph[i * 2] = -magnitude * 10 + h
	end
	love.graphics.line(graph)

	love.graphics.print(#fft, 400)
end